@@ -20,6 +20,10 @@
 #include <iostream>
 #include <sstream>
 #include <cstdlib>
+#include <stdlib.h>
+#include <unistd.h>
+
+
 
 extern "C" {
 #ifdef HAVE_GETOPT_H
@@ -41,6 +45,8 @@ extern "C" {
 #include <signal.h>
 #include <pthread.h>
 #include <semaphore.h>
+
+#include <X11/Xlib.h>
 }
 
 #include <lineak/definitions.h>
@@ -58,6 +64,7 @@ extern "C" {
 //#include "commandexec.h"
 
 extern bool verbose;
+extern bool very_verbose;
 
 using namespace lineak_util_functions;
 using namespace std;
@@ -73,6 +80,16 @@ void lineak_core_functions::fatal(const string message) {
    lineak_core_functions::fatal(message.c_str());
 }
 
+void lineak_core_functions::vmsg(const string message) {
+	   lineak_core_functions::vmsg(message.c_str());
+}
+void lineak_core_functions::verror(const string message) {
+	   lineak_core_functions::verror(message.c_str());
+}
+void lineak_core_functions::vfatal(const string message) {
+	   lineak_core_functions::vfatal(message.c_str());
+}
+
 void lineak_core_functions::msg(const char* message) {
     if (verbose) {
        cout << message << endl;
@@ -92,6 +109,25 @@ void lineak_core_functions::fatal(const char* message) {
     }
 }
 
+void lineak_core_functions::vmsg(const char* message) {
+    if (very_verbose) {
+        cout << message << endl;
+    }
+}
+void lineak_core_functions::verror(const char* message) {
+    if (very_verbose) {
+        cerr << message << endl;
+    }
+}
+void lineak_core_functions::vfatal(const char* message) {
+    if (very_verbose) {
+        cerr << "Fatal Error: " << message << endl;
+        msgPasser message;
+        message.start();
+        message.sendMessage(msgPasser::EXIT,"exit");
+    }
+}
+
 /* Parse the configuration file.
 		prefs: is a ConfigDirectives object. It is loaded with values obtained from the command line, and
 			other system directives and defaults.
@@ -131,28 +167,28 @@ bool lineak_core_functions::parseconffile(ConfigDirectives & prefs, LConfig & my
 // prefs at this point is made up of first: directives and defaults from all of the plugins.
 // second: any values specified on the command line are added to or overwrite the values in the directives and defaults.
 // Now, we will pass this to the configloader, who will likewise overwrite and
-   msg("Instantiating ConfigLoader");
+   vmsg("Instantiating ConfigLoader");
    msg("Loading a config file");
    //prefs.print(cout);
    msg("conffilename = :" + conffilename);
    ConfigLoader configldr(conffilename, prefs);
-   //cout << "Loading a config file.\n";
+   //cout << "Loading a config file. Got a configldr\n";
    //(configldr.loadConfig()).print(cout);
    //cout << "Doing it again.\n" << endl;
-   configldr >> myConfig;
-   //myConfig = configldr.loadConfig();
+   //configldr >> myConfig;
+   myConfig = configldr.loadConfig();
    //sleep(2);
    msg("Displaying config.");
    if (verbose) 
 	   myConfig.print(cout);
-   msg("Checking to see if it is empty");
-
+   
+   vmsg("Checking to see if it is empty");
    if (myConfig.isEmpty()) {
 		cerr << "*** Configuration file " << conffilename << " could not be loaded" << endl;
 		cerr << "		Have you defined actions for your keys?" << endl;
 		return(false);
    }
-   msg("Returning from parseconffile");
+   vmsg("Returning from parseconffile");
    return (true); 
 }
 
@@ -176,7 +212,7 @@ bool lineak_core_functions::parsedeffile(ConfigDirectives & prefs, LDef & def) {
 		usrdeffilename = homedir + LINEAKDIR;
 		usrdeffilename += DEFFILE;
    }
-   if (verbose) msg("Parsing: " + usrdeffilename);
+   msg("Parsing: " + usrdeffilename);
    if (file_exists(usrdeffilename)) {
       /* it exists, parse the contents */
       DefLoader defldr(usrdeffilename);
@@ -194,7 +230,7 @@ bool lineak_core_functions::parsedeffile(ConfigDirectives & prefs, LDef & def) {
    }
    
    if ( deffilename != usrdeffilename ) {
-      if (verbose) msg("Parsing: " + deffilename);
+      msg("Parsing: " + deffilename);
       /* does the file exist? */
       if (file_exists(deffilename)) {
 	/* it exists, parse the contents */
@@ -217,8 +253,8 @@ bool lineak_core_functions::parsedeffile(ConfigDirectives & prefs, LDef & def) {
      /** Append the user definition data to the system data */
      def.addKeyboards(udef);
   }
-  //msg("Outputting the definition file!");
-  //if (verbose) cout << def;
+  vmsg("Outputting the definition file!");
+  if (very_verbose) cout << def;
    return (true);
 }
 /*
@@ -463,7 +499,7 @@ string lineak_core_functions::getTypeString(KeyType_t type) {
 unsigned int lineak_core_functions::getModifierNumericValue(const string modstr) {
 	string modifiers = modstr;
 	unsigned int modifier = 0;
-	int j = 0;
+        string::size_type j = 0;
 	vector<string> mods;
 	
 	// Add a '+' to the end so we have something to parse on in the 
@@ -473,7 +509,8 @@ unsigned int lineak_core_functions::getModifierNumericValue(const string modstr)
 	}
 	// Break the string up and put the individual modifiers into a vector 
 	// of strings.
-	while (((unsigned int)(j = modifiers.find('+'))) != string::npos) {
+	while (modifiers.find('+') != string::npos) {
+		j = modifiers.find('+');
 		//cout << "adding modifier: " << modifiers.substr(0,j) << " to key:" << keyname << endl;
 		mods.push_back(modifiers.substr(0,j));
 		modifiers.erase(0,j+1);
@@ -527,3 +564,42 @@ unsigned int lineak_core_functions::getModifierNumericValue(const string modstr)
 	}
 	return modifier;
 }
+
+void lineak_core_functions::enable_IBMRA7993(void) {
+   lineak_core_functions::send_commands("ea 71");
+}
+
+void lineak_core_functions::send_commands(string args) {
+/*   int x;
+
+   string st;
+   istringstream SString( args );
+
+   if (ioperm(0x60, 3, 1) == -1) {
+      perror("ioperm failed.");
+   } 
+
+   while ( SString >> st ) {
+      if (verbose) cout << "Running command: " << st << endl;
+
+      x = strtol(st.c_str(),0,16);
+      usleep(300);
+      outb(x, 0x60);
+   }
+   */
+  string comm = SBINDIR;
+  comm += "/send_to_keyboard ";
+  comm += args;
+  comm += " &";
+
+  if (!fork()) {
+
+        /* child process that tries to run the command, and then exits */
+        /* all specials done, let's go for it... ;) */
+        if (verbose) cout << "... running " << comm << endl;
+        system(comm.c_str());
+        exit(true);
+  }
+
+}
+
