@@ -51,7 +51,7 @@ bool PluginManager::loadPlugin(const string& plugin) {
         initialize_t initialize;
         cleanup_t cleanup;
 	identifier_t identifier;
-	identifier_info*plugin_ident;
+	identifier_info *plugin_ident;
 	directivelist_t directive_list;
 	macrolist_t macro_list;
 
@@ -63,6 +63,7 @@ bool PluginManager::loadPlugin(const string& plugin) {
 	string name;
 	string type;
 	string version;
+        string description;
 
         void *handle;
         const char *errmsg;
@@ -90,7 +91,7 @@ bool PluginManager::loadPlugin(const string& plugin) {
 	// Attempt to get the plugin identifier. This will prevent us from loading the same plugin more than once.
 	identifier = (identifier_t)dlsym(handle, "identifier");
 	if((errmsg = dlerror()) != NULL) {
-		error("Didn't find identifier() interface: " + string(errmsg));
+		verror("Didn't find identifier() interface: " + string(errmsg));
 //		show = NULL;
 //		volume = NULL;
                 get_display = NULL;
@@ -100,18 +101,24 @@ bool PluginManager::loadPlugin(const string& plugin) {
 		return false;
 	} else {
 		plugin_ident = identifier();
-		name = string(plugin_ident->identifier);
-		type = string(plugin_ident->type);
-		version = string(plugin_ident->version);
+		name = plugin_ident->identifier;
+		type = plugin_ident->type;
+		version = plugin_ident->version;
+                description = plugin_ident->description;
                 msg("plugin name:" + name);
                 msg("plugin type:" + type);
 		msg("plugin version: " + version); 
-		if (VERSION != version)
-			msg("Warning: Version mismatch. In the future this error will exit the daemon.");
+		msg("plugin_ident: " + description);
+		if (VERSION != version) {
+			msg("Warning: Version mismatch. Unloading plugin.");
+                        dlerror();
+                        dlclose(handle);
+                        return false;
+                }
 		if (type == "MACRO"  || type == "DISPLAY" && (name != "" && name != snull))  {
         		// Check to see if that plugin is already registered. If so, skip to the next one.
-			if (plugin_map.find((plugin_ident->identifier)) != plugin_map.end()) {
-				error("Plugin: " + string(plugin_ident->identifier) + " is already registered!");
+			if (hasPlugin(name)) {
+				error("Plugin: " + name + " is already registered!");
 				dlerror();
 				dlclose(handle);
 				return false;
@@ -129,7 +136,7 @@ bool PluginManager::loadPlugin(const string& plugin) {
 	// on certain versions of Linux and across platform.
 	initialize = (initialize_t)dlsym(handle,"initialize");
 	if((errmsg = dlerror()) != NULL) {
-		error("Didn't find initalize() interface: " + string(errmsg));
+		verror("Didn't find initalize() interface: " + string(errmsg));
 		initialize = NULL;
 		dlerror();
 		dlclose(handle);
@@ -140,7 +147,7 @@ bool PluginManager::loadPlugin(const string& plugin) {
 	// Attempt to resolve the cleanup interface for the plugin.
 	cleanup = (cleanup_t)dlsym(handle,"cleanup");
 	if((errmsg = dlerror()) != NULL) {
-		error("Didn't find cleanup() interface: " + string(errmsg));
+		verror("Didn't find cleanup() interface: " + string(errmsg));
 		cleanup = NULL;
 		dlerror();
 		dlclose(handle);
@@ -148,10 +155,10 @@ bool PluginManager::loadPlugin(const string& plugin) {
 	}
 	dlerror();
 
-	// Attempt to initialize the xosd portion of the plugin.
+	// Attempt to initialize the osd portion of the plugin.
 	initialize_display = (initialize_display_t)dlsym(handle,"initialize_display");
 	if((errmsg = dlerror()) != NULL) {
-		error("Didn't find initalize_display() interface: " + string(errmsg));
+		verror("Didn't find initalize_display() interface: " + string(errmsg));
 		initialize_display = NULL;
 	}
 	dlerror();
@@ -174,7 +181,7 @@ bool PluginManager::loadPlugin(const string& plugin) {
 
         get_display = (get_display_t)dlsym(handle, "get_display");
 	if((errmsg = dlerror()) != NULL) {
-		error("Didn't find get_display interface: "+ string(errmsg));
+		verror("Didn't find get_display interface: "+ string(errmsg));
 		get_display = NULL;
 		if (type == "DISPLAY") {
 			error("DISPLAY plugin: " + name + "could not load the volume interface");
@@ -201,7 +208,7 @@ bool PluginManager::loadPlugin(const string& plugin) {
 	// Attempt to open the exec interface.
 	exec = (exec_t)dlsym(handle, "exec");
 	if((errmsg = dlerror()) != NULL) {
-		error("Didn't find exec() interface: " + string(errmsg));
+		verror("Didn't find exec() interface: " + string(errmsg));
 		exec = NULL;
 		if (type == "MACRO") {
 			error("MACRO plugin: " + name + "could not load the exec interface");
@@ -214,7 +221,7 @@ bool PluginManager::loadPlugin(const string& plugin) {
 	// Attempt to open the macrolist interface
 	directive_list = (directivelist_t)dlsym(handle,"directivelist");
 	if((errmsg = dlerror()) != NULL) {
-		error("Didn't find directivelist() interface: " + string(errmsg));
+		verror("Didn't find directivelist() interface: " + string(errmsg));
 		directive_list = NULL;
 		//dlerror();
 		//dlclose(handle);
@@ -224,7 +231,7 @@ bool PluginManager::loadPlugin(const string& plugin) {
 	// Attempt to open the macrolist interface
 	macro_list = (macrolist_t)dlsym(handle,"macrolist");
 	if((errmsg = dlerror()) != NULL) {
-		error("Didn't find macrolist() interface: " + string(errmsg));
+		verror("Didn't find macrolist() interface: " + string(errmsg));
 		macro_list = NULL;
 		if (type == "MACRO") {
 			error("MACRO plugin: " + name + "could not open the macrolist interface. This means we cannot get the list of macros that this plugin supports. Closing plugin.");
@@ -258,16 +265,18 @@ bool PluginManager::loadPlugin(const string& plugin) {
 	plinfo.initialized = false;
 	plinfo.macros.clear();
 	plinfo.directives.clear();
-
-	plugin_map[string(plugin_ident->identifier)] = plinfo;
+        if (verbose) cout << "Entering plugin: " << name << endl;
+//	if (verbose) cout << "plinfo ---------" << endl;
+//	if (verbose) cout << plinfo << endl;
+	plugin_map[name] = plinfo;
 
 	return true;
 }
 vector<string> PluginManager::loadPlugins(vector<string> myplugins) {
 	string plugin;
 	vector<string> loadedplugins;
-        unsigned int size = plugin_map.size();
-        unsigned int numplugins = myplugins.size();
+        map<string, plugin_info>::size_type size = plugin_map.size();
+        vector<string>::size_type numplugins = myplugins.size();
 	//bool loadflag = true;
 
 	if (myplugins.empty()) {
@@ -293,8 +302,9 @@ vector<string> PluginManager::loadPlugins(vector<string> myplugins) {
 }
 /** Unload a plugin */
 void PluginManager::unloadPlugin(const string& plugin) {
-	void* handle;
-	cleanup_t cleanup;
+	msg("unloadPlugin: enter for plugin " + plugin);
+	void* handle = NULL;
+	cleanup_t cleanup = NULL;
 	msg("PluginManager: cleaning up plugin " + plugin);
 	handle = plugin_map[plugin].handle;
 	cleanup = plugin_map[plugin].cleanup;
@@ -304,17 +314,20 @@ void PluginManager::unloadPlugin(const string& plugin) {
         if (handle != NULL) {
                 dlclose(handle);
         }
+	msg("PluginManager: removing plugin " + plugin);
 	plugin_map.erase(plugin);
-
+        msg("UnloadPlugin: exit");
 }
 /** Unloads all loaded plugins. */
 void PluginManager::unloadAllPlugins(){
+        string name = "";
    	map<string,plugin_info>::iterator it = plugin_map.begin();
         msg("PluginManager is unloading plugins");
 	for (;it != plugin_map.end(); it++) {
-		unloadPlugin(it->first);
+                if (verbose) cout << "Calling unload for plugin: " << it->first << endl;
+		//unloadPlugin(it->first);
 	}
-	//msg("PluginManager finished unloading plugins");
+	msg("PluginManager finished unloading plugins");
         // Remove all plugin entries in the map.
 	plugin_map.clear();
 	msg("PluginManager unloading completed");
@@ -323,7 +336,8 @@ void PluginManager::unloadAllPlugins(){
 /** Initialize a particular plugin */
 bool PluginManager::initializePlugin(const string& plugin_ident, LKbd & imyKbd, LConfig & imyConfig, PluginManager *iplugins) {
 	/** Check to see that the plugin is loaded */
-	if(plugin_ident != "" && plugin_ident != snull) {
+	if(plugin_ident != "" && plugin_ident != snull && 
+	   (hasPlugin(plugin_ident))) {
 		if (plugin_map[plugin_ident].loaded == true) {
 			initialize_t initialize;
 			msg("Initializing Plugin:" + plugin_ident);
@@ -384,6 +398,8 @@ bool PluginManager::initializePlugins(LKbd & imyKbd, LConfig & imyConfig,PluginM
 bool PluginManager::initializePluginDisplay(const string& plugin, displayCtrl & imyDisplay) {
 	/** Check to see that the plugin is loaded */
 	if(plugin != "" && plugin != snull) {
+		if (plugin_map.find(plugin) == plugin_map.end())
+			return false;
 		if (plugin_map[plugin].loaded == true ) {
 			msg("Initializing OSD for plugin: " + plugin);
         		initialize_display_t initialize_display;
@@ -415,6 +431,7 @@ bool PluginManager::initializePluginsDisplay(displayCtrl & imyDisplay) {
         map<string,plugin_info>::iterator it = plugin_map.begin();
 	bool initflag = true;
         for (;it != plugin_map.end(); it++) {
+		if (verbose) cout << "InitializePluginsDisplay = " << it->first << endl;
         	if(!initializePluginDisplay((it->first), imyDisplay))
 			initflag = false;
         }
@@ -467,7 +484,7 @@ bool PluginManager::defineMacroList(const string& plugin) {
 			macrolistfn = info.macrolist;
 			// Insurance
 			if (macrolistfn != NULL) {
-				char **array;
+				string *array;
 				string tmp;
 				macinfo = macrolistfn();
 				if (macinfo == NULL) {
@@ -479,7 +496,7 @@ bool PluginManager::defineMacroList(const string& plugin) {
 				// If it is not a MACRO plugin, just return. It is not an error for a non-macro plugin to
 				// not hanve any macros defined.
 				if (macinfo->num_macros == 0) {
-					if (string(info.identifier->type) == "MACRO") {
+					if (info.identifier->type == "MACRO") {
 						unloadPlugin(plugin);
 						return false;
 					}
@@ -657,19 +674,24 @@ exec_t PluginManager::exec(LObject* imyKey, XEvent xev) {
         return NULL;
 }
 displayCtrl* PluginManager::getDisplay(const LConfig &config) {
-        string displayp = config.getValue("Display_plugin");
-        if (displayp == DEFAULT_DISPLAY_PLUGIN)
-                return (NULL);
-        // Search for the plugin that corresponds to "Display_plugin" and call it's get_display function.
-        else {
-                plugin_info plinfo = plugin_map[displayp];
-                if (plinfo.filename != "" && plinfo.initialized && plinfo.loaded) {
-                        get_display_t get_display = plinfo.get_display;
-                        if ( get_display != NULL)
-                                return (get_display());
-                }
-        }
-        return (NULL);
+    string displayp = config.getValue("Display_plugin");
+    //displayCtrl *rdisplay = NULL;
+
+    if (displayp == DEFAULT_DISPLAY_PLUGIN)
+       return (NULL);
+    //Search for the plugin that corresponds to "Display_plugin" and call it's get_display function.
+    else if (hasPlugin(displayp)) {
+       plugin_info plinfo = plugin_map[displayp];
+       if (plinfo.filename != "" && plinfo.initialized && plinfo.loaded) {
+          get_display_t get_display = plinfo.get_display;
+          if ( get_display != NULL)
+             return (get_display());
+       }
+    }
+    else
+       cout << "Display plugin not found" << endl;
+
+    return (NULL);
 }
 void PluginManager::print() {
 	map<string,plugin_info>::iterator it = plugin_map.begin();
@@ -684,3 +706,17 @@ void PluginManager::print() {
 		}
 	}
 }
+void PluginManager::listPlugins() {
+	cout << "Listing Plugins" << endl;
+	map<string,plugin_info>::iterator it = plugin_map.begin();
+	for (; it != plugin_map.end(); it++) {
+	   cout << "Plugin " << it->first << " of type " << ((it->second).identifier)->type << endl;
+           cout << "  Description: " << ((it->second).identifier)->description << endl;
+        }
+	cout << endl;
+}
+bool PluginManager::hasPlugin(const string &plugin) {
+      if (plugin_map.find(plugin) != plugin_map.end())
+	  return true;
+      return false; 
+}
