@@ -118,7 +118,7 @@ void lineakd_core_functions::showusage(int argc, char *argv[]) {
 	cout << "    -c TYPE,   --config=TYPE        Create a new lineakd.conf file for keyboard TYPE\n";
 	cout << "                                      (warning: old one will be overwritten!)\n";
 	cout << "    -v,        --verbose            Print verbose messages\n";
-	cout << "                                      (only visible when not running in background)\n";
+	cout << "    -vv,       --very-verbose       Print very verbose messages. Mostly for debugging purposes.\n";
 	cout << "    -h,        --help               Print this message\n";
 	cout << "    -r,        --reload             Reload the configuration files.\n";
         cout << "    -z         --sleep              Disable handling of keyboard events\n";
@@ -147,7 +147,7 @@ void lineakd_core_functions::showusage(int argc, char *argv[]) {
 	cout << "    -c TYPE       Create a new lineakd.conf file for TYPE keyboard\n";
 	cout << "                    (warning: old one will be overwritten!)\n";
 	cout << "    -v,           Print verbose messages\n";
-	cout << "                    (only visible when not running in background)\n";
+	cout << "    -vv,          Print very verbose messages. Mostly for debugging purposes.\n";
 	cout << "    -h            Print this message\n";
 	cout << "    -r,           Reload the configuration files.\n";
         cout << "    -z           Disable handling of keyboard events\n";
@@ -270,9 +270,9 @@ void lineakd_core_functions::do_exit() {
         if (!plugincleanup) {
            plugincleanup = true;
            if ( plugins != NULL) {
-              //msg("Unloading All Plugins!");
-              //plugins->unloadAllPlugins();
-              //msg("Unloading All Plugins DONE!");
+              msg("Unloading All Plugins!");
+              plugins->unloadAllPlugins();
+              msg("Unloading All Plugins DONE!");
               delete(plugins);
               plugins = NULL;
               msg("Plugins all taken care of!");
@@ -325,38 +325,42 @@ void lineakd_core_functions::restart() {
                 msg("Unmapping keys");
                 X->cleanup(myKbd);
 
-                //msg("Unloading all plugins");
-                //plugins->unloadAllPlugins();
+                msg("Unloading all plugins");
+                plugins->unloadAllPlugins();
+
                 /** Get a list of the potential plugins in the plugin directory */
-                //vector<string> pluginlist = plugins->scanForPlugins();
-                //msg("Scaning for and loading plugins");
+                vector<string> pluginlist = plugins->scanForPlugins();
+                msg("Scaning for and loading plugins");
                 /** Load the plugins */
-                //plugins->loadPlugins(pluginlist);
-		
+                plugins->loadPlugins(pluginlist);
                 /** Define the list of macros we support */
                 plugins->defineMacroLists();
                 /** Define the configurate directives we support */
                 plugins->defineDirectivesLists();
                 /** Get a list of macros we support so that we can inform other classes */
                 vector<string> macrolist = plugins->getMacroList();
+
                 /** Get a list of directives and their defaults that we support here */
                 dnd = plugins->getDirectivesList();
 
                 /** Add to the list of directives and defaults that we support implicitly. */
-                dnd.addValue(_CD_KEYBOARD_TYPE, snull);
-                dnd.addValue(_CD_USERCONFFILE, snull);
-                dnd.addValue(_CD_CDROM_DEVICE, DEFAULT_CDROM_DEVICE);
-                dnd.addValue(_CD_MIXER_DEVICE, DEFAULT_MIXER_DEVICE);
-                dnd.addValue("Screensaver", snull);
-                dnd.addValue("Display_plugin",DEFAULT_DISPLAY_PLUGIN);
-                dnd.addValue("Display_font", DISPLAY_FONT);
-                dnd.addValue("Display_color", DISPLAY_COLOR);
-                dnd.addValue("Display_pos", DISPLAY_POS);
-                dnd.addValue("Display_align", DISPLAY_ALIGN);
-                dnd.addValue("Display_timeout", DISPLAY_TIMEOUT);
-                dnd.addValue("Display_hoffset", DISPLAY_HOFFSET);
-                dnd.addValue("Display_voffset", DISPLAY_VOFFSET);
-                dnd.addValue("Display_soffset", DISPLAY_SOFFSET);
+	        if (!dnd.isSet(_CD_KEYBOARD_TYPE))    dnd.addValue(_CD_KEYBOARD_TYPE, snull);
+                if (!dnd.isSet(_CD_USERCONFFILE))     dnd.addValue(_CD_USERCONFFILE, snull);
+	        if (!dnd.isSet(_CD_CDROM_DEVICE))     dnd.addValue(_CD_CDROM_DEVICE, DEFAULT_CDROM_DEVICE);
+	        if (!dnd.isSet(_CD_MIXER_DEVICE))     dnd.addValue(_CD_MIXER_DEVICE, DEFAULT_MIXER_DEVICE);
+	        if (!dnd.isSet("Screensaver"))        dnd.addValue("Screensaver", snull);
+                if (!dnd.isSet("Display_plugin"))     dnd.addValue("Display_plugin",DEFAULT_DISPLAY_PLUGIN);
+                if (!dnd.isSet("Display_font"))       dnd.addValue("Display_font", DISPLAY_FONT);
+	        if (!dnd.isSet("Display_color"))      dnd.addValue("Display_color", DISPLAY_COLOR);
+	        if (!dnd.isSet("Display_pos"))        dnd.addValue("Display_pos", DISPLAY_POS);
+                if (!dnd.isSet("Display_align"))      dnd.addValue("Display_align", DISPLAY_ALIGN);
+                if (!dnd.isSet("Display_timeout"))    dnd.addValue("Display_timeout", DISPLAY_TIMEOUT);
+                if (!dnd.isSet("Display_hoffset"))    dnd.addValue("Display_hoffset", DISPLAY_HOFFSET);
+                if (!dnd.isSet("Display_voffset"))    dnd.addValue("Display_voffset", DISPLAY_VOFFSET);
+                if (!dnd.isSet("Display_soffset"))    dnd.addValue("Display_soffset", DISPLAY_SOFFSET);
+                if (!dnd.isSet("keystate_numlock"))   dnd.addValue("keystate_numlock", snull);
+                if (!dnd.isSet("keystate_capslock"))  dnd.addValue("keystate_capslock", snull);
+                if (!dnd.isSet("keystate_scrolllock")) dnd.addValue("keystate_scrolllock", snull);
 
                 /** Set the macrolist for all LCommand objects */
                 LCommand::setMacros(macrolist);
@@ -618,3 +622,122 @@ void *lineakd_core_functions::handle_messages(void *arg) {
 	}
   pthread_exit(NULL);
 }
+void lineakd_core_functions::print_event(int event) {
+   switch(event) {
+     case KeyPress:
+         cout << "KeyPress: Event" << endl;
+         break;
+     case KeyRelease:
+         cout << "KeyRelease: Event" << endl;
+         break;
+     case ButtonPress:
+         cout << "ButtonPress: Event" << endl;
+         break;
+     case ButtonRelease:
+         cout << "ButtonRelease: Event" << endl;
+         break;
+     case MotionNotify:
+         cout << "MotionNotify: Event" << endl;
+         break;
+     case EnterNotify:
+         cout << "EnterNotify: Event" << endl;
+         break;
+     case LeaveNotify:
+         cout << "LeaveNotify: Event" << endl;
+         break;
+     case FocusIn:
+         cout << "FocusIn: Event" << endl;
+         break;
+     case FocusOut:
+         cout << "FocusOut: Event" << endl;
+         break;
+     case KeymapNotify:
+         cout << "KeymapNotify: Event" << endl;
+         break;
+     case Expose:
+         cout << "Expose: Event" << endl;
+         break;
+     case GraphicsExpose:
+         cout << "GraphicsExpose: Event" << endl;
+         break;
+     case NoExpose:
+         cout << "NoExpose: Event" << endl;
+         break;
+     case VisibilityNotify:
+         cout << "VisibilityNotify: Event" << endl;
+         break;
+     case CreateNotify:
+         cout << "CreateNotify: Event" << endl;
+         break;
+     case DestroyNotify:
+         cout << "DestroyNotify: Event" << endl;
+         break;
+     case UnmapNotify:
+         cout << "UnmapNotify: Event" << endl;
+         break;
+     case MapNotify:
+         cout << "MapNotify: Event" << endl;
+         break;
+     case MapRequest:
+         cout << "MapRequest: Event" << endl;
+         break;
+     case ReparentNotify:
+         cout << "ReparentNotify: Event" << endl;
+         break;
+     case ConfigureNotify:
+         cout << "ConfigureNotify: Event" << endl;
+         break;
+     case ConfigureRequest:
+         cout << "ConfigureRequest: Event" << endl;
+         break;
+     case GravityNotify:
+         cout << "GravityNotify: Event" << endl;
+         break;
+     case ResizeRequest:
+         cout << "ResizeRequest: Event" << endl;
+         break;
+     case CirculateNotify:
+         cout << "CirculateNotify: Event" << endl;
+         break;
+     case CirculateRequest:
+         cout << "CirculateRequest: Event" << endl;
+         break;
+     case PropertyNotify:
+         cout << "PropertyNotify: Event" << endl;
+         break;
+     case SelectionClear:
+         cout << "SelectionClear: Event" << endl;
+         break;
+     case SelectionRequest:
+         cout << "SelectionRequest: Event" << endl;
+         break;
+     case SelectionNotify:
+         cout << "SelectionNotify: Event" << endl;
+         break;
+     case ColormapNotify:
+         cout << "ColormapNotify: Event" << endl;
+         break;
+     case ClientMessage:
+         cout << "ClientMessage: Event" << endl;
+         break;
+     case MappingNotify:
+         cout << "MappingNotify: Event" << endl;
+         break;
+     case LASTEvent:
+         cout << "LASTEvent: Event" << endl;
+         break;
+     default:
+         cout << "Unknown: Event" << endl;
+         break;
+   }
+}
+void lineakd_core_functions::initialize_raw_keyboards(LConfig & config, LDef & def) {
+
+   LKbd & kb = def.getKeyboard(config[_CD_KEYBOARD_TYPE]);
+   string coms = kb.getRawCommand("INITIALIZE");
+
+   if ( (! kb.isEmpty()) && coms.length() > 0 ) {
+      send_commands(coms);
+   }
+
+}
