@@ -45,13 +45,13 @@
 extern "C" {
 #include <signal.h>
 #include <X11/Xlib.h>
-#include <X11/Xlibint.h>
+//#include <X11/Xlibint.h>
 #include <X11/Xutil.h>
-#include <X11/Xatom.h>
-#include <X11/Intrinsic.h>
-#include <X11/Xos.h>
-#include <X11/Xmu/Error.h>
-#include <X11/Xmu/Xmu.h>
+//#include <X11/Xatom.h>
+//#include <X11/Intrinsic.h>
+//#include <X11/Xos.h>
+//#include <X11/Xmu/Error.h>
+//#include <X11/Xmu/Xmu.h>
 }
 
 #ifndef X_NOT_STDC_ENV
@@ -60,11 +60,6 @@ extern "C" {
   extern char *getenv();
 #endif
 
-
-//#ifdef HAVE_XOSD
- // #include "xosdctrl.h"
-//	xosdCtrl *myDisplay = 0;
-//#endif
 #include <lineak/displayctrl.h>
 displayCtrl *myDisplay = 0;
 
@@ -103,6 +98,7 @@ CmdPrefs cmdprefs;
 vector<string> macrolist;
 ConfigDirectives dnd;
 extern bool verbose;
+extern bool very_verbose;
 extern bool global_enable;
 
 int main(int argc, char *argv[])
@@ -116,7 +112,10 @@ int main(int argc, char *argv[])
        exit(false);
 
   verbose = cmdprefs.getVerbose();
+  very_verbose = cmdprefs.getVeryVerbose();
+
   if (verbose) cout << "Lineakd V" << VERSION << endl;
+  if (very_verbose) cout << "Lineakd running in very verbose mode.!" << endl;
 
   if (cmdprefs.showUsage()) {
        showusage(argc, argv);
@@ -128,19 +127,31 @@ int main(int argc, char *argv[])
         showkeyboards(myDef);
         exit(true);
   }
-  
   /** Get a list of the potential plugins in the plugin directory */
   plugins = new PluginManager;
   vector<string> pluginlist = plugins->scanForPlugins();
 
   /** Load the plugins */
   plugins->loadPlugins(pluginlist);
+  if (verbose) {
+	  msg("");
+	  msg("Listing Plugins");
+	  plugins->listPlugins();
+  }
+
+  msg("");
+  msg("Defining Macro Lists");
   /** Define the list of macros we support */
   plugins->defineMacroLists();
+  //if (verbose) plugins->listPlugins();
   /** Define the configurate directives we support */
+  msg("");
+  msg("Defining Directives Lists");
   plugins->defineDirectivesLists();
+  //if (verbose) plugins->listPlugins();
    /** Get a list of macros we support so that we can inform other classes */
   macrolist = plugins->getMacroList();
+  //if (verbose) plugins->listPlugins();
 
   /* If all we have to do is look for plugins and macros */
   if (cmdprefs.showPlugins()) {
@@ -154,11 +165,11 @@ int main(int argc, char *argv[])
   dnd = plugins->getDirectivesList();
 
   /** Add to the list of directives and defaults that we support implicitly. */
-  /** Right now we are stomping all over plugin defined directives! FIXME */
    if (!dnd.isSet(_CD_KEYBOARD_TYPE))    dnd.addValue(_CD_KEYBOARD_TYPE, snull);
    if (!dnd.isSet(_CD_USERCONFFILE))     dnd.addValue(_CD_USERCONFFILE, snull);
    if (!dnd.isSet(_CD_CDROM_DEVICE))     dnd.addValue(_CD_CDROM_DEVICE, DEFAULT_CDROM_DEVICE);
    if (!dnd.isSet(_CD_MIXER_DEVICE))     dnd.addValue(_CD_MIXER_DEVICE, DEFAULT_MIXER_DEVICE);
+   if (!dnd.isSet(_CD_RAW_COMMANDS))     dnd.addValue(_CD_RAW_COMMANDS, snull);
    if (!dnd.isSet("Screensaver"))        dnd.addValue("Screensaver", snull);
    if (!dnd.isSet("Display_plugin"))     dnd.addValue("Display_plugin",DEFAULT_DISPLAY_PLUGIN);
    if (!dnd.isSet("Display_font"))       dnd.addValue("Display_font", DISPLAY_FONT);
@@ -207,28 +218,33 @@ int main(int argc, char *argv[])
   else
   	lock.lock();
   */
-  /** WARINING!!! End of Displayless functions. A reference to the display required for
-  		function from here on. */
+
+  /**************************************************************************************************
+      WARINING!!! End of Displayless functions. A reference to the display required for
+  		  proper functioning from this point on. 
+
+   **************************************************************************************************/
+
   if (!X) {
-   	X = new(Xmgr);
-  }
-  // Temporarily commeted out while tracking key bug.
-  if (!X->initialized())
+     X = new(Xmgr);
+     if ((!X->initialized()) || (X->getDisplay() == NULL)) {
+        cerr << "Could not open display: " << getenv("DISPLAY") << endl;
   	cleanexit(false);
-  /** WARNING!!! From here on in, you should be using cleanexit() if you need to exit without
-  		worrying about threading issues.
-
-  		cleanexit will take care of deleting the X and myDisplay objects, as well as removing
-    		the message queue and unlocking.
-
-    		Once watch_messages is called, we should be calling do_exit() instead, to deal with
-     		threading issues.
-
-     **/
-  if (!X->getDisplay()) {
-   	cerr << "Could not open the display. Exiting." << endl;
-   	cleanexit(false);
+     }
   }
+
+  /**************************************************************************************************
+      WARNING!!! From here on in, you should be using cleanexit() if you need to exit without
+  		 concern for threading issues.
+
+  		 cleanexit will take care of deleting the X and myDisplay objects, as well as removing
+    		 the message queue and unlocking.
+
+      WARNING!!! Once watch_messages is called, we should be calling do_exit() instead, to deal with
+     		 threading issues.
+
+    *************************************************************************************************/
+
   /** Create the users .lineak directory */
   create_homedir();
   /* everything went according to plan thisfar... some signal-handlers for cleaner exiting */
@@ -246,7 +262,6 @@ int main(int argc, char *argv[])
    *  the config file options. */
   cmdprefs.setOpts(myConfig);
   parseDefinitions(cmdprefs, myDef);
-//  cout << myConfig << endl;
   /* build our EAKeyboard with the data from deffile and conffile */
   /** get the keyboard from our config file */
   myKbd = myDef.getKeyboard(myConfig[_CD_KEYBOARD_TYPE]);
@@ -258,9 +273,13 @@ int main(int argc, char *argv[])
    	cleanexit(false);
   }
   /** Update our keyboard with our configured commands */
-   myKbd.setCommands(myConfig);
+  myKbd.setCommands(myConfig);
+
+  /** Send appropriate control codes to the keyboard. */
+  initialize_raw_keyboards(myConfig,myDef);
+
   /* 
-   if (verbose) {
+   if (very_verbose) {
     cout << myKbd;
     for (map<string,LObject*>::const_iterator i = myKbd.getObjects().begin(); i != myKbd.getObjects().end(); i++) {
       LObject *m = const_cast<LObject*>(i->second);
@@ -282,7 +301,7 @@ int main(int argc, char *argv[])
 	  error("Could not establish IPC via the message queue.");
 	  fatal("Please resolve the problem and retry");
 	  cleanexit(false);
-  }
+  } 
   /** Setup the message watching thread. This thread responds to stop and restart messages.
       From here on out, we should use do_exit() to quit the program */
   watch_messages();
@@ -303,11 +322,14 @@ int main(int argc, char *argv[])
 
   /* initialize OSD system */
   myDisplay = plugins->getDisplay(myConfig);
+
   if (myDisplay != NULL) {
         myDisplay->init();
   } else {
         myDisplay = new displayCtrl(myConfig);
+        myDisplay->init();
   }
+  msg("Initializing Plugin Display");
   plugins->initializePluginsDisplay(*myDisplay);
 
   /* alright, we're going to loop forever now. only signals can interrupt us. */
@@ -322,14 +344,61 @@ int main(int argc, char *argv[])
   exec.setDisplayCtrl(myDisplay);
 
   while(true) {
+     if (very_verbose) cout << "Waiting for next event." << endl;
      /* wait for next event */
      XNextEvent(display, &xev);
+     if (very_verbose) cout << "Got an event." << endl;
      /* Should we age the display? */
-     if (myDisplay->enabled()) {
-         //cout << "agenscroll()" << endl;
-         myDisplay->agenscroll();
+     if (myDisplay != NULL) {
+        if ((xev.type != MappingNotify) && myDisplay->enabled()) {
+            //cout << "agenscroll()" << endl;
+            myDisplay->agenscroll();
+        }
      }
+     // If something is taking the keyboard, or it is being remapped.
+     // Relinquish our hold on the keyboard mappings.
+     // And shutdown the on screen display.
+     // This is something needed to make it play nicely with things
+     // like VMWare.
+     if (xev.type == MappingNotify) {
+//       if (!X->xkbRemapped()) {
+           msg("Setting xkbRemapped flag");
+           X->xkbRemapped(true); // =1;
+           msg("Cleaning up key mappings from MappingNotify event");
+           X->cleanup(myKbd);
+	   msg("Cleaning up the on screen display.");
+	   myDisplay->cleanup();
+//       }
+       // If we are getting the display back.
+       // Get out modifiers and grab our keys again.
+       // Reinitialize the OSD.
+//       else {
+           msg("Clearing xkbRemapped flag");
+           X->xkbRemapped(false);
+           //X->getModifiers();
+           msg("Reclaiming keyboard map from MappingNotify event");
+           X->initialize(myKbd);
+	   msg("Restarting the OSD.");
+	   myDisplay = plugins->getDisplay(myConfig);
+	   if (myDisplay != NULL) {
+	       if (very_verbose) cout << "Reiniting plugin provided On Screen Display" << endl;
+	       
+	       myDisplay->init();
+	   } else {
+	       if (very_verbose) cout << "Initializing internal display." << endl;
+	       myDisplay = new displayCtrl(myConfig);
+	       myDisplay->init();
+	   }
+	   // Initialize the plugin displays again.
+	   msg("Initializing Plugin Display");
+	   plugins->initializePluginsDisplay(*myDisplay);
+	   // set exec.setDisplayCtrl(myDisplay) again.
+	   exec.setDisplayCtrl(myDisplay);
+//        }
+     }
+     if (very_verbose) cout << "Doing something with it!" << endl;
      if (global_enable) {
+        if (very_verbose) cout << "Global Enabled" << endl;
         switch (xev.type)
 	{
 	   case KeyPress:
@@ -337,7 +406,7 @@ int main(int argc, char *argv[])
 	         
 	         xev.xkey.state &= ~(numlock_mask | capslock_mask | scrolllock_mask);
 	         LKey *obj = static_cast<LKey*>(myKbd.getObject((int)xev.xkey.keycode));
-		 if (verbose) {
+		 if (very_verbose) {
 		    cout << "Key press !" << endl;
 		    cout << "xev.xkey.keycode=" << xev.xkey.keycode << endl;
 		    cout << "xev.xkey.state=" << xev.xkey.state << endl;
@@ -347,13 +416,13 @@ int main(int argc, char *argv[])
 	         if (obj != NULL) {
 		    msg("Got an object!");
 	            if (obj->getType() == SYM && obj->getEventType() == PRESS) {
-		       if (xev.xkey.keycode == XKeysymToKeycode (display, obj->getKeySym())
+		      cout << "xev.xkey.keycode = " << xev.xkey.keycode << endl;
+		      cout << "obj->getKeyCode() = " << obj->getKeyCode() << endl;
+		      cout << "XKeysymToKeycode (display, (KeySym)obj->getKeySym() = " << XKeysymToKeycode (display, (KeySym)obj->getKeySym()) << endl;
+		       if (xev.xkey.keycode == XKeysymToKeycode (display, (KeySym)obj->getKeySym())
 		           && obj->hasModifier(xev.xkey.state))
 		       {
 		          execute = plugins->exec(obj, xev);
-			  //TODO: execute and exec will need to be modified to pass the
-			  // xev through. Then in these functions, the correct command
-			  // for the modifier will have to be chosen.
 	                  if (execute != NULL)
                              execute(obj,xev);
 			  else 
@@ -399,18 +468,19 @@ int main(int argc, char *argv[])
 	   } 
 	   case KeyRelease:
 	   {
-	      /* if (verbose) {
-	         cout << "Key press !" << endl;
+	       xev.xkey.state &= ~(numlock_mask | capslock_mask | scrolllock_mask);
+	       LKey *obj = static_cast<LKey*>(myKbd.getObject((int)xev.xkey.keycode));
+
+	       if (very_verbose) {
+	         cout << "Key release !" << endl;
 		 cout << "xev.xkey.keycode=" << xev.xkey.keycode << endl;
 		 cout << "xev.xkey.state=" << xev.xkey.state << endl;
-	      } */
+		 cout << "has modifier=" << obj->hasModifier(xev.xkey.state) << endl;
+	       } 
 
-	      xev.xkey.state &= ~(numlock_mask | capslock_mask | scrolllock_mask);
-	      LKey *obj = static_cast<LKey*>(myKbd.getObject((int)xev.xkey.keycode));
-		 
 	      if (obj != NULL) { 
 	         if (obj->getType() == SYM && obj->getEventType() == RELEASE) {
-		    if (xev.xkey.keycode == XKeysymToKeycode (display, obj->getKeySym())
+		    if (xev.xkey.keycode == XKeysymToKeycode (display, (KeySym)obj->getKeySym())
 		        && obj->hasModifier(xev.xkey.state)) {
 		       execute = plugins->exec(obj,xev);
 	               if (execute != NULL)
@@ -447,16 +517,18 @@ int main(int argc, char *argv[])
 	   } 
 	   case ButtonPress:
 	   {
-	        /* if (verbose) {
+                LButton *obj = static_cast<LButton*>(myKbd.getObject((unsigned int)xev.xbutton.button));
+		
+	        if (very_verbose) {
 	           cout << "Button press !" << endl;
 	           cout << "xev.xbutton.button=" << xev.xbutton.button << endl;
 	           cout << "xev.xbutton.state=" << xev.xbutton.state << endl;
-	        } */
+		   cout << "has modifier=" << obj->hasModifier(xev.xkey.state) << endl;
+	        } 
 
 	        xev.xbutton.state &= ~(numlock_mask | capslock_mask | scrolllock_mask
 			       | Button1Mask | Button2Mask | Button3Mask
 			       | Button4Mask | Button5Mask);
-	        LButton *obj = static_cast<LButton*>(myKbd.getObject((unsigned int)xev.xbutton.button));
                 if (obj != NULL) {
 	           if (obj->getType() == BUTTON && obj->getEventType() == PRESS) {
 		      if (xev.xbutton.button == obj->getButton() && obj->hasModifier(xev.xbutton.state))
@@ -479,16 +551,20 @@ int main(int argc, char *argv[])
            }
 	   case ButtonRelease:
 	   {
-	     /* if (verbose) {
-	        cout << "Button press !" << endl;
+
+             LButton *obj = static_cast<LButton*>(myKbd.getObject((unsigned int)xev.xbutton.button));
+
+	     if (very_verbose) {
+	        cout << "Button release !" << endl;
 	        cout << "xev.xbutton.button=" << xev.xbutton.button << endl;
 	        cout << "xev.xbutton.state=" << xev.xbutton.state << endl;
-	     } */
+		cout << "has modifier=" << obj->hasModifier(xev.xkey.state) << endl;
+	     }
 
 	     xev.xbutton.state &= ~(numlock_mask | capslock_mask | scrolllock_mask
 			       | Button1Mask | Button2Mask | Button3Mask
 			       | Button4Mask | Button5Mask);
-	     LButton *obj = static_cast<LButton*>(myKbd.getObject((unsigned int)xev.xbutton.button));
+
              if (obj != NULL) {
 	     if (obj->getType() == BUTTON && obj->getEventType() == RELEASE)
 		{
@@ -511,26 +587,24 @@ int main(int argc, char *argv[])
 	    }
 	     break;
 	  }
-	  case MappingNotify: {
-		if (!X->xkbRemapped())
-		   X->xkbRemapped(true); // =1;
-		else {
-		   msg("Reclaiming keyboard map from MappingNotify event");
-		   X->xkbRemapped(false);
-		   //X->getModifiers();
-		   X->initialize(myKbd);
-		}
+	   default:
 	     break;
-	  }
-	  default:
-	     break;
-	}
+        } // End switch
+        if (very_verbose)
+	     lineakd_core_functions::print_event(xev.type);
+        
+     } // End global_enable
+     else {
+	     if (very_verbose) cout << "Global is not enabled." << endl;
      }
-     if (myDisplay->enabled()) {
+
+     if (myDisplay != NULL) {
+        if (myDisplay->enabled()) {
 	   //msg("age()");
-        myDisplay->age();
+           myDisplay->age();
+        }
      }
-  }
+  } // End while(true) loop.
   /* that's all folks.. We should never get here!!! */
   do_exit();
   return true;
